function taskFinish(task)
% TASKFINISH Perform user-specific task cleanup actions.
%
%   taskFinish(task)
%
%   To define specific task cleanup actions on each worker
%   for each task, you can do any of the following:
%   1. Add m-code that performs those actions to this file for each worker.
%   2. Add to the job's PathDependencies property a directory that
%      contains a file named taskFinish.m.
%   3. Include a file named taskFinish.m in the job's FileDependencies
%      property.
%
%   The file in FileDependencies takes precendence over the
%   PathDependencies file, which takes precedence over this file on
%   the worker's installation.
%
%   The task parameter that is passed to this function is the task object
%   that the worker has just executed.
%
%   This function runs even if the jobStartup, taskStartup, or the
%   task itself threw an error.
%
%   If this function throws an error, the error information appears in the task's
%   ErrorMessage and ErrorIdentifier properties unless the jobStartup,
%   taskStartup, or the task itself threw an error.
%
%   Any path changes made here or during the execution of tasks will be
%   reverted by the MATLAB Distributed Computing Server to their original
%   values before the next job runs, but preserved for subsequent tasks in
%   the same job.  Any data stored by this function or by the execution of
%   the job's tasks (for example, in the base workspace or in global or
%   persistent variables) will not be cleared by the MATLAB Distributed
%   Computing Server before the next job runs, unless the RestartWorker
%   property of the next job is set to true.
%
%   See also jobStartup, taskStartup.

%fprintf('Modified TaskFinish(%d) Started!\n',task.ID);
%cacschedulerLog(sprintf('Task finish started %d',task.ID));

% handle legacy name
isZip = getenv('CACSCHEDULER_ZIP_JOB');
if isempty(isZip)
    isZip = getenv('LITTLEJOHN_ZIP_JOB');
end
isNew = getenv('CACSCHEDULER_ZIP_VERSION');
if ~isempty(isZip)
    %Only one task should execute this, everyone else goes on!
    j = task.Parent;
    ts = get(j,'Tasks');
    nTasks = numel(ts);
    if findstr('matlabpooljob',class(j)) 
        zipTask = 1;
    elseif isempty(isNew)
        zipTask = 1; %backward compatible
    else
        zipTask = numel(ts);
    end
    if (get(task,'ID')) ~= zipTask
%       fprintf('Not zip task, passing right along\n');
    else
        tCount = nTasks-1;
        complete = 0;
        while complete < tCount
            complete = length(strmatch('finished',...
                get(ts,'State')));
            %exception for matlabpooljob, task1 must finish first
            if findstr('matlabpooljob',class(j))
                if task.ID == 1
                    complete = tCount;
                else
                    return;
                end
            end
        end
        %everybody but me is now complete, so figure the zip!

        jobDir = get(j,'Name');
        storageLocation = getenv( 'MDCE_STORAGE_LOCATION');
        jobLocation = getenv( 'MDCE_JOB_LOCATION' );
        taskLocation = getenv('MDCE_TASK_LOCATION' );
        jobDirPath = [storageLocation '/' jobLocation];

        % manually update to finish
        task.pSetState('finished');

        %Set job.state.mat to finished
        j.pSetState('finished');

        % update finish time
        endTime = char(java.util.Date);
        serializer = j.pReturnSerializer;
        serializer.putFields(j, {'finishtime' 'state'}, ...
            {endTime 'finished'});
        serializer.putFields(task, {'finishtime' 'state'}, ...
            {endTime 'finished'});
        % for pool job update worker states
        if findstr('matlabpooljob',class(j))
            for t=1:nTasks
                serializer.putFields(ts(t), {'finishtime' 'state'}, ...
                    {endTime 'finished'});
            end
        end
        %Now zip the goods.
        filesToZip={};
        filesToZip{end+1} = jobDirPath;
        filesToZip{end+1} = [storageLocation '/' jobLocation ...
            '.*'];
        % handle legacy name
        zipLocation = getenv('LITTLEJOHN_ZIP_LOCATION');
        if isempty(zipLocation)
            zipLocation = getenv('CACSCHEDULER_ZIP_LOCATION');
        end
%       fprintf('Creating %s\n',zipLocation);
        zip(zipLocation ,filesToZip);
    end
end

% Copyright 2010 Cornell Center for Advanced Computing
% Modified by Dr. Nate Woody
% Modified by Eric Chen
% Modified by Steven Clark
